#include <basic.h>
#include <tcode.h>
#include <new>
#include "subc.h"
#include "kdither.h"
#include "kim_data.h"
#include "kpixel.h"
#include "kpalet.h"

extern OutSider grovalFunc;


KDither::KDither(KPallet *pl)
{
  int n = pl->get_colorNum();
  colorPallet = new(std::nothrow) KPallet(n);
  for (int i = 0; i < n; i++){
    colorPallet->put_color(i, pl->get_color(i));
  }
}

int KDither::getNearYPbPr(KYPbPr &d)
{
  int n = colorPallet->srch_nearYPbPrColor(d);
  KRGB rgb = colorPallet->get_color(n);
  d = rgb.get_YPbPr();
	
  return n;
}

unsigned char *KDither::calc_ditherFloyd(unsigned char *dst, KImageData *src)
{
  grovalFunc.println((TC[]){L"桡"});
  int sizeH = src->get_sizeH();
  int sizeV = src->get_sizeV();
// cout << "sizeH = " << sizeH << endl;
// cout << "sizeV = " << sizeV << endl;
  KYPbPr *noise = new(std::nothrow) KYPbPr[sizeH * 2];
	
  for (int i = 0; i < sizeH; i++){
    KYPbPr d0, d1;
    src->get_data(i, 0, d1);
    if (i != 0){
      d1.YY += noise[i - 1].YY;
      d1.Pb += noise[i - 1].Pb;
      d1.Pr += noise[i - 1].Pr;
    }
    d0 = d1;
    dst[i] = getNearYPbPr(d0);
    noise[i].YY = d1.YY - d0.YY;
    noise[i].Pb = d1.Pb - d0.Pb;
    noise[i].Pr = d1.Pr - d0.Pr;
//		src->put_data(i, 0, d0);
    if ((noise[i].YY > 255) || (noise[i].YY < -255)){
      noise[i].YY = 0;
      noise[i].Pb = 0;
      noise[i].Pr = 0;
    }
  }
	
  for (int j = 1; j < sizeV; j++){
    for (int i = 0; i < sizeH; i++){
      if (i == 0){
        KYPbPr d0, d1;
        src->get_data(0, j, d1);
        int pos = (j % 2) * sizeH;
        d0 = d1;
        dst[j * sizeH + i] = getNearYPbPr(d0);
        noise[pos].YY = d1.YY - d0.YY;
        noise[pos].Pb = d1.Pb - d0.Pb;
        noise[pos].Pr = d1.Pr - d0.Pr;
//				src->put_data(0, j, d0);
        continue;
      }
      KYPbPr d7, d8, d9, d12;
      KYPbPr d00, d01;
      KYPbPr buff;
			
      int line_pos = (j % 2) * sizeH;
      int preLine_pos = ((j - 1) % 2 ) * sizeH;
      d7 = noise[i-1 + preLine_pos];
      d8 = noise[i   + preLine_pos];
      if ((i + 1) < sizeH){
        d9 = noise[i+1 + preLine_pos];
      }
      else {
        d9 = d8;
      }
      d12 = noise[i-1 + line_pos];
			
      src->get_data(i, j, d01);
      buff.YY = d7.YY * 1 + d8.YY * 5 + d9.YY * 3 + d12.YY * 7;
      buff.YY /= 16;
      d01.YY += buff.YY;
			
      buff.Pb = d7.Pb * 1 + d8.Pb * 5 + d9.Pb * 3 + d12.Pb * 7;
      buff.Pb /= 16;
      d01.Pb += buff.Pb;
			
      buff.Pr = d7.Pr * 1 + d8.Pr * 5 + d9.Pr * 3 + d12.Pr * 7;
      buff.Pr /= 16;
      d01.Pr += buff.Pr;
			
      long pos = line_pos + i;
      d00 = d01;
      dst[j * sizeH + i] = getNearYPbPr(d00);
      d01.YY -= d00.YY;
      d01.Pb -= d00.Pb;
      d01.Pr -= d00.Pr;
      noise[pos] = d01;
//			src->put_data(i, j, d00);
      if ((noise[pos].YY > 255) || (noise[pos].YY < -255)){
        noise[pos].YY = 0;
        noise[pos].Pb = 0;
        noise[pos].Pr = 0;
      }
    }
    if ((j % 10) == 0){
      grovalFunc.println((TC[]){L""});
    }
  }
  grovalFunc.println((TC[]){TK_BSLH, TK_n, 0});
	
  delete [] noise;
  return 0;
}





/*

int interporationHV(unsigned char *data, int size_h, int size_v)
{
	for(int x = 0; x < size_h; x += 2){
		for(int y = 0; y < size_v; y += 2){
			int pos0 = y       * size_h + x;
			int pos1 = (y + 1) * size_h + x;
			int pos2 = (y + 2) * size_h + x;
			int p0 = (int)data[pos0];
			int p1 = (int)data[pos2];
			data[pos0] = (unsigned char)((p0 * 3 + p1) / 4);
			data[pos1] = (unsigned char)((p0 + p1 * 3) / 4);
		}
	}
	for(int y = 0; y < size_v; y++){
		for(int x = 0; x < size_h; x += 2){
			int pos0 = y * size_h + x;
			int pos1 = pos0 + 1;
			int pos2 = pos0 + 2;
			int p0 = (int)data[pos0];
			int p1 = (int)data[pos2];
			data[pos0] = (unsigned char)((p0 * 3 + p1) / 4);
			data[pos1] = (unsigned char)((p0 + p1 * 3) / 4);
		}
	}
	return 0;
}


int interporationHV(KImageData444 *data)
{
	interporationHV(data->get_pPb(), data->get_sizeH(), data->get_sizeV());
	interporationHV(data->get_pPr(), data->get_sizeH(), data->get_sizeV());
	
	return 0;
}




static int TAD16[16][3] = { {255, 255, 255},
							{  0,   0, 159},
							{  0, 239,   0},
							{  0, 239, 239},
							{255,   0,   0},
							{239,   0, 255},
							{223, 223,   0},
							{127, 127, 159},
							{223, 223, 223},
							{127, 159, 255},
							{191, 255, 175},
							{207, 255, 255},
							{255, 111, 111},
							{239, 143, 255},
							{239, 255, 159},
							{  0,   0,   0}	
						};


int main(int argc, char *argv[])
{
	if(argc != 3){
		cout << "Usage : kdither <in YUV file name> <out BMP file name>" << endl;
		return 1;
	}
	
	KPallet pl(16);
	for(int ii = 0; ii < 16; ii++){
		pl.put_color(ii, KRGB(TAD16[ii][0], TAD16[ii][1], TAD16[ii][2]));
	}
	
	KDither dither(&pl);
	
	
	int fp = ::open(argv[1], O_RDONLY | O_BINARY);
	if(fp < 0){
		cout << "Can't open file" << endl;
		return 1;
	}
	
	int sizeH2 = 320;
	int sizeH1 = 480;
	int sizeV = 240;
	long sizeY = sizeH1 * sizeV;
	long sizeP = sizeH2 * sizeV / 4;
	
	
	unsigned char *buff = new unsigned char[sizeY + sizeP * 2];
	unsigned char *org_p = buff;
	
	::lseek(fp, 0L, 0);
	for(int i = 0; i < 960; i++){
		::read(fp, org_p, 160);
		org_p += 160;
	}
	::close(fp);
	
	
	
	KImageData444 *in_data = new KImageData444(sizeH2, sizeV);
	unsigned char *buffYY = in_data->get_pYY();
	unsigned char *buffPb = in_data->get_pPb();
	unsigned char *buffPr = in_data->get_pPr();
	
	unsigned char *yy = buff;
	unsigned char *pb = yy + sizeY;
	unsigned char *pr = pb + sizeP;
	
	//	load to YY memory
	for(int y = 0; y < sizeV; y++){
		for(int x = 0; x < sizeH1; x++){
			if((x % 3) != 0){
				*buffYY++ = *yy;
			}
			yy++;
		}
	}
	
	// load to Pb and Pr memories
	for(y = 0; y < (sizeV / 2); y++){
		for(int x = 0; x < (sizeH2 / 2); x++){
			int aa = (int)*pb++;
			int bb = (int)*pr++;
			*buffPb = (unsigned char)(aa + 128);
			*buffPr = (unsigned char)(bb + 128);
			buffPb += 2;
			buffPr += 2;
		}
		buffPb += sizeH2;
		buffPr += sizeH2;
	}
	
	interporationHV(in_data);
	
	dither.calc_ditherFloyd(buff, in_data);
	
	KBitmap24 *out_data = new KBitmap24(sizeH2, sizeV);
	
	for(y = 0; y < sizeV; y++){
		KRGB d;
		for(int x = 0; x < sizeH2; x++){
			long pos = y * sizeH2 + x;
//			d = pl.get_color(buff[pos]);
in_data->get_data(x, y, d);
			out_data->put_data(x, y, d);
		}
	}

	KBitmapFile24 test;
	test.open(argv[2], sizeH2, sizeV);
	test.write_data(out_data);
//	test.prt_data();
	test.close();
	
	delete out_data;
	delete in_data;
	delete[] buff;
	
	return 0;
}
*/
