#include <basic.h>
#include <bstdio.h>
#include <new>
#include "kim_data.h"
#include "kpalet.h"
#include "ketc.h"


// class KImageData abstract class functions

KImageData::~KImageData()
{
  ;
}

int KImageData::checkXY(int x, int y)
{
  if ((x < 0) || (x >= sizeH) || (y < 0) || (y >= sizeV)){
    return 1;
  }
  return 0;
}

int KImageData::marume(int &x, int &y)
{
  int ret = 0;
	
  if (x < 0){
    x = 0;
    ret++;
  }
  else if (x > sizeH){
    x = sizeH;
    ret++;
  }
  if (y < 0){
    y = 0;
    ret++;
  }
  else if (y > sizeV){
    y = sizeV;
    ret++;
  }
  return(ret);
}


// class KImageDataRGB class functions

KImageDataRGB::KImageDataRGB(int h_size, int v_size)
  : KImageData(h_size, v_size)
{
  dataRR   = new(std::nothrow) unsigned char [h_size * v_size];
  dataGG   = new(std::nothrow) unsigned char [h_size * v_size];
  dataBB   = new(std::nothrow) unsigned char [h_size * v_size];
  dataFlg = _K_RGB;
}

KImageDataRGB::~KImageDataRGB()
{
  delete[] dataRR;
  delete[] dataGG;
  delete[] dataBB;
}

int KImageDataRGB::get_data(int x, int y, KRGB& d)
{
  if (checkXY(x, y) != 0){
    return 1;
  }
	
  int pos = getPos(x, y);
	
  d.RR = dataRR[pos];
  d.GG = dataGG[pos];
  d.BB = dataBB[pos];
  return 0;
}

int KImageDataRGB::get_data(int x, int y, KYPbPr& d)
{
  if (checkXY(x, y) != 0){
    return 1;
  }
	
  KRGB org;
  this->get_data(x, y, org);
  d = org.get_YPbPr();
	
  return 0;
}

int KImageDataRGB::put_data(int x, int y, KRGB d)
{
  if (checkXY(x, y) != 0){
    return 1;
  }
	
  int pos = getPos(x, y);
	
  clipping(d.RR, 0, 255);
  clipping(d.GG, 0, 255);
  clipping(d.BB, 0, 255);
  dataRR[pos] = (unsigned char)d.RR;
  dataGG[pos] = (unsigned char)d.GG;
  dataBB[pos] = (unsigned char)d.BB;
	
  return 0;
}

int KImageDataRGB::put_data(int x, int y, KYPbPr d)
{
  if (checkXY(x, y) != 0){
    return 1;
  }
	
  KRGB data;
  data = d.get_RGB();
  clipping(data.RR, 0, 255);
  clipping(data.GG, 0, 255);
  clipping(data.BB, 0, 255);
  return this->put_data(x, y, data);
}



// class KImageData444 class functions

KImageData444::KImageData444(int h_size, int v_size)
  : KImageData(h_size, v_size)
{
  dataYY = NULL;
  dataPb = NULL;
  dataPr = NULL;
  newData(h_size, v_size);
  dataFlg = _K_YPbPr444;
}

KImageData444::~KImageData444()
{
  delData();
}

void KImageData444::newData(int h_size, int v_size)
{
//cout << "444 data create!!" << endl;
  delData();
  dataYY = new(std::nothrow) unsigned char [h_size * v_size];
  if (dataYY == NULL){
    printf("++++++++++++++++dataYY out of memory+++++++++++++++++\n");
  }
  dataPb = new(std::nothrow) unsigned char [h_size * v_size];
  if(dataPb == NULL){
    printf("++++++++++++++++dataPb out of memory+++++++++++++++++\n");
  }
  dataPr = new(std::nothrow) unsigned char [h_size * v_size];
  if(dataPr == NULL){
    printf("++++++++++++++++dataPr out of memory+++++++++++++++++\n");
  }
}

void KImageData444::delData(void)
{
//cout << "data delete" << endl;
  if (dataYY != NULL){
    delete[] dataYY;
    dataYY = NULL;
  }
  if (dataPb != NULL){
    delete[] dataPb;
    dataPb = NULL;
  }
  if (dataPr != NULL){
    delete[] dataPr;
    dataPr = NULL;
  }
}



int KImageData444::get_data(int x, int y, KYPbPr& d)
{
  if(checkXY(x, y) != 0){
    return 1;
  }
	
  int posY = getYPos(x, y);
  int posP = getPPos(x, y);
	
  d.YY = dataYY[posY];
  d.Pb = dataPb[posP] - 128;
  d.Pr = dataPr[posP] - 128;
	
  return 0;
}

int KImageData444::get_data(int x, int y, KRGB& d)
{
  if (checkXY(x, y) != 0){
    return 1;
  }
	
  KYPbPr org;
  this->get_data(x, y, org);
  d = org.get_RGB();
	
  return 0;
}

int KImageData444::put_data(int x, int y, KYPbPr d)
{
  if (checkXY(x, y) != 0){
    return 1;
  }
	
  int posY = getYPos(x, y);
  int posP = getPPos(x, y);
	
  clipping(d.YY, 0, 255);
  clipping(d.Pb, -128, 127);
  clipping(d.Pr, -128, 127);
  dataYY[posY] = d.YY;
  dataPb[posP] = d.Pb + 128;
  dataPr[posP] = d.Pr + 128;
	
  return 0;
}

int KImageData444::put_data(int x, int y, KRGB d)
{
  if (checkXY(x, y) != 0){
    return 1;
  }
	
  KYPbPr data;
  data = d.get_YPbPr();
	
  return this->put_data(x, y, data);
}



// class KImageData422 class functions

KImageData422::KImageData422(int h_size, int v_size)
  : KImageData444(h_size, v_size)
{
  newData(h_size, v_size);
  dataFlg = _K_YPbPr422;
}

void KImageData422::newData(int h_size, int v_size)
{
//cout << "422 data create!!" << endl;
  delData();
  dataYY = new(std::nothrow) unsigned char [h_size * v_size];
  dataPb = new(std::nothrow) unsigned char [h_size / 2 * v_size];
  dataPr = new(std::nothrow) unsigned char [h_size / 2 * v_size];
}


// class KImageData420 class functions

KImageData420::KImageData420(int h_size, int v_size)
  : KImageData444(h_size, v_size)
{
  newData(h_size, v_size);
  dataFlg = _K_YPbPr420;
}

void KImageData420::newData(int h_size, int v_size)
{
//cout << "420 data create!!" << endl;
  delData();
  dataYY = new(std::nothrow) unsigned char [h_size * v_size];
  dataPb = new(std::nothrow) unsigned char [h_size / 2 * v_size / 2];
  dataPr = new(std::nothrow) unsigned char [h_size / 2 * v_size / 2];
}


// class KImageDataBW class functions

KImageDataBW::KImageDataBW(int h_size, int v_size)
  : KImageData(h_size, v_size)
{
  dataBW   = new(std::nothrow) unsigned char [h_size * v_size];
  dataFlg = _K_BW;
}

KImageDataBW::~KImageDataBW()
{
  delete[] dataBW;
}

int KImageDataBW::get_data(int x, int y, KRGB& d)
{
  if (checkXY(x, y) != 0){
    return 1;
  }
	
  KYPbPr org;
  this->get_data(x, y, org);
  d = org.get_RGB();
	
  return 0;
}

int KImageDataBW::get_data(int x, int y, KYPbPr& d)
{
  if (checkXY(x, y) != 0){
    return 1;
  }
	
  int pos = getPos(x, y);
	
  d.YY = dataBW[pos];
  d.Pb = 0;
  d.Pr = 0;
	
  return 0;
}

int KImageDataBW::put_data(int x, int y, KRGB d)
{
  if (checkXY(x, y) != 0){
    return 1;
  }
	
  KYPbPr data;
  data = d.get_YPbPr();
	
  return this->put_data(x, y, data);
}

int KImageDataBW::put_data(int x, int y, KYPbPr d)
{
  if (checkXY(x, y) != 0){
    return 1;
  }
	
  int pos = getPos(x, y);
	
  clipping(d.YY, 0, 255);
  dataBW[pos] = (unsigned char)d.YY;
	
  return 0;
}



// class KBitmap24 class functions

KBitmap24::KBitmap24(int h_size, int v_size)
  : KImageData(h_size, v_size)
{
  data = new(std::nothrow) unsigned char [h_size * v_size * 3];
  dataFlg = _K_BITMAP24;
}

KBitmap24::~KBitmap24()
{
  delete[] data;
}

int KBitmap24::get_data(int x, int y, KRGB& d)
{
  if (checkXY(x, y) != 0){
    return 1;
  }
	
  int pos = getPos(x, y);
	
  d.BB = data[pos];
  d.GG = data[pos + 1];
  d.RR = data[pos + 2];
	
  return 0;
}

int KBitmap24::get_data(int x, int y, KYPbPr& d)
{
  if (checkXY(x, y) != 0){
    return 1;
  }
	
  KRGB org;
  this->get_data(x, y, org);
  d = org.get_YPbPr();
	
  return 0;
}

int KBitmap24::put_data(int x, int y, KRGB d)
{
  if (checkXY(x, y) != 0){
    return 1;
  }
	
  int pos = getPos(x, y);
  clipping(d.RR, 0, 255);
  clipping(d.GG, 0, 255);
  clipping(d.BB, 0, 255);
	
  data[pos    ] = d.BB;
  data[pos + 1] = d.GG;
  data[pos + 2] = d.RR;
	
  return 0;
}

int KBitmap24::put_data(int x, int y, KYPbPr d)
{
  KRGB rgb;
  rgb = d.get_RGB();
  
  return this->put_data(x, y, rgb);
}



// class KBitmap4 class functions

KBitmap4::KBitmap4(int h_size, int v_size, KPallet *pl)
  : KImageData(h_size, v_size)
{
  byte_sizeH = (h_size / 2) + (h_size % 2);
  data = new(std::nothrow) unsigned char [byte_sizeH * sizeV];
  dataFlg = _K_BITMAP4;
  colorPallet = new(std::nothrow) KPallet(pl->get_colorNum());
  for (int i = 0; i < pl->get_colorNum(); i++){
    colorPallet->put_color(i, pl->get_color(i));
  }
}

KBitmap4::~KBitmap4()
{
  delete[] data;
  delete colorPallet;
}

int KBitmap4::get_data(int x, int y, KRGB& d)
{
  if (checkXY(x, y) != 0){
    return 1;
  }
	
  int pos = getPos(x, y);
	
  unsigned char c = data[pos];
  if ((x % 2) == 0){
    c = c >> 4;
  }
  c &= 0x0f;
  d = colorPallet->get_color((int)c);
	
  return 0;
}

int KBitmap4::get_data(int x, int y, KYPbPr& d)
{
  if (checkXY(x, y) != 0){
    return 1;
  }
	
  KRGB org;
  this->get_data(x, y, org);
  d = org.get_YPbPr();
	
  return 0;
}

int KBitmap4::get_data(int x, int y, int& col)
{
  if (checkXY(x, y) != 0){
    return 1;
  }
	
  int pos = getPos(x, y);
	
  unsigned char c = data[pos];
  if ((x % 2) == 0){
    c = c >> 4;
  }
  c &= 0x0f;
  col = (int)c;
	
  return 0;
}

int KBitmap4::put_data(int x, int y, KRGB d)
{
  if (checkXY(x, y) != 0){
    return 1;
  }

  int col_num = colorPallet->srch_color(d);
  if (col_num < 0){
    col_num = colorPallet->srch_nearRGBColor(d);
  }
	
  long int pos = getPos(x, y);
  unsigned char c, mask;
	
  c = col_num & 0x000f;
  if ((x % 2) == 0){
    c = c << 4;
    mask = 0x0f;
  }
  else{
    mask = 0xf0;
  }
	
  data[pos] = (data[pos] & mask) | c;
	
  return 0;
}

int KBitmap4::put_data(int x, int y, KYPbPr d)
{
  if (checkXY(x, y) != 0){
    return 1;
  }
  
  KRGB rgb;
  rgb = d.get_RGB();
	
  return this->put_data(x, y, rgb);
}

int KBitmap4::put_data(int x, int y, int col)
{
  if (checkColorNum(col) != 0){
    return 1;
  }
	
  return put_data(x, y, colorPallet->get_color(col));
}

int KBitmap4::checkColorNum(int n)
{
  if ((n < 0) || (n >= colorPallet->get_colorNum())){
    return 1;
  }
  return 0;
}



/*
int  main(int argc, char *argv[])
{
	int fp = ::open(argv[1], O_RDONLY);
	if(fp < 0){
//		cout << "Can't open file" << endl;
		return 1;
	}
	
	int sizeH2 = 360;
	int sizeH1 = 480;
	int sizeV = 240;
	
	KImageData420 *in_data = new KImageData420(sizeH2, sizeV);
	unsigned char *buffYY = in_data->get_pYY();
	unsigned char *buffPb = in_data->get_pPb();
	unsigned char *buffPr = in_data->get_pPr();
	
	unsigned char *yy = new unsigned char[sizeH1 * sizeV];
	
	::lseek(fp, 0L, 0);
	::read(fp, yy, sizeH1 * sizeV);
	::read(fp, buffPb, sizeH2 * sizeV / 4);
	::read(fp, buffPr, sizeH2 * sizeV / 4);
	::close(fp);
	
	unsigned char *p1 = buffYY;
	unsigned char *p2 = yy;
	for(long i = 0; i < (sizeH1 * sizeV); i++){
		if((i % 3) != 0){
			*p1++ = *p2;
		}
		p2++;
	}
	
	KBitmap24 *out_data = new KBitmap24(sizeH2, sizeV);
	for(int y = 0; y < sizeV; y++){
		for(int x = 0; x < sizeH2; x++){
			KYPbPr d;
			in_data->get_data(x, y, d);
			out_data->put_data(x, y, d);
		}
	}
	
	delete out_data;
	delete in_data;
	delete[] yy;
	
	return 0;
}
*/
