//
//		KImageData
//			KImageDataRGB
//			KImageData444
//				KImageData422
//				KImageData420
//			KImageDataBW
//			KMitmap24
//

#ifndef	_K_IMAGE_DATA_H
#define	_K_IMAGE_DATA_H

#include "kpixel.h"
#include "kpalet.h"

enum KDataFlg {
  _K_RGB, _K_BW, 
  _K_YPbPr420, _K_YPbPr422, _K_YPbPr444, 
  _K_BITMAP24, _K_BITMAP4
};

class KImageData
{
protected:
  int sizeH, sizeV;
  KDataFlg dataFlg;
	
public:
  KImageData(int h_size, int v_size){
    sizeH = h_size;
    sizeV = v_size;
  }
  virtual ~KImageData();
	
  int get_sizeH() {
    return sizeH;
  }
  int get_sizeV() {
    return sizeV;
  }
  KDataFlg get_type() {
    return dataFlg;
  }
	
  virtual int get_data(int x, int y, KYPbPr& d) = 0;
  virtual int get_data(int x, int y, KRGB& d) = 0;
  virtual int put_data(int x, int y, KYPbPr d) = 0;
  virtual int put_data(int x, int y, KRGB d) = 0;

protected:
  int checkXY(int x, int y);
  int marume(int& x, int& y);
};


class KImageDataRGB
  : public KImageData
{
  unsigned char* dataRR;
  unsigned char* dataGG;
  unsigned char* dataBB;

public:
  KImageDataRGB(int h_size, int v_size);
  virtual ~KImageDataRGB();
	
  virtual int get_data(int x, int y, KYPbPr& d);
  virtual int get_data(int x, int y, KRGB& d);
  virtual int put_data(int x, int y, KYPbPr d);
  virtual int put_data(int x, int y, KRGB d);
	
  unsigned char *get_pRR() {
    return dataRR;
  }
  unsigned char *get_pBB() {
    return dataBB;
  }
  unsigned char *get_pGG() {
    return dataGG;
  }
	
protected:
  int getPos(int x, int y) {
    return (y * sizeH + x);
  }
};



class KImageData444
  : public KImageData
{
protected:
  unsigned char *dataYY;
  unsigned char *dataPb;
  unsigned char *dataPr;
	
public:
  KImageData444(int h_size, int v_size);
  virtual ~KImageData444();
	
  virtual int get_data(int x, int y, KYPbPr& d);
  virtual int get_data(int x, int y, KRGB& d);
  virtual int put_data(int x, int y, KYPbPr d);
  virtual int put_data(int x, int y, KRGB d);
	
  unsigned char *get_pYY() {
    return dataYY;
  }
  unsigned char *get_pPb() {
    return dataPb;
  }
  unsigned char *get_pPr() {
    return dataPr;
  }
	
protected:
  virtual long getYPos(int x, int y) {
    return (y * sizeH + x);
  }
  virtual long getPPos(int x, int y) {
    return (y * sizeH + x);
  }
  virtual void newData(int h_size, int v_size);
  void delData();
};


class KImageData422
  : public KImageData444
{
public:
  KImageData422(int h_size, int v_size);
  virtual ~KImageData422() {}
	
protected:
  virtual long getYPos(int x, int y) {
    return (y * sizeH + x);
  }
  virtual long getPPos(int x, int y) {
    return (y * sizeH / 2 + (int)(x / 2));
  }
  virtual void newData(int h_size, int v_size);
};


class KImageData420
  : public KImageData444
{
public:
  KImageData420(int h_size, int v_size);
  virtual ~KImageData420() {}
	
protected:
  virtual long getYPos(int x, int y) {
    return (y * sizeH + x);
  }
  virtual long getPPos(int x, int y) {
    return ((int)(y / 2) * sizeH / 2 + (int)(x / 2));
  }
  virtual void newData(int h_size, int v_size);
};


class KImageDataBW
  : public KImageData
{
  unsigned char *dataBW;

public:
  KImageDataBW(int h_size, int v_size);
  virtual ~KImageDataBW();
	
  virtual int get_data(int x, int y, KYPbPr& d);
  virtual int get_data(int x, int y, KRGB& d);
  virtual int put_data(int x, int y, KYPbPr d);
  virtual int put_data(int x, int y, KRGB d);
	
  unsigned char *get_pBW() {
    return dataBW;
  }
	
protected:
  int getPos(int x, int y) {
    return (y * sizeH + x);
  }
};


class KBitmap24
  : public KImageData
{
  unsigned char *data;

public:
  KBitmap24(int h_size, int v_size);
  virtual ~KBitmap24();
	
  virtual int get_data(int x, int y, KYPbPr& d);
  virtual int get_data(int x, int y, KRGB& d);
  virtual int put_data(int x, int y, KYPbPr d);
  virtual int put_data(int x, int y, KRGB d);
	
  unsigned char *get_pBMP() {
    return data;
  }
	
protected:
  int getPos(int x, int y) {
    return (((sizeV - y - 1) * sizeH + x) * 3);
  }
	
};


class KBitmap4
  : public KImageData
{
private:
  int byte_sizeH;
  unsigned char *data;
  KPallet *colorPallet;

public:
  KBitmap4(int h_size, int v_size, KPallet *pl);
  virtual ~KBitmap4();
	
  virtual int get_data(int x, int y, KYPbPr& d);
  virtual int get_data(int x, int y, KRGB& d);
  virtual int get_data(int x, int y, int& col);
  virtual int put_data(int x, int y, KYPbPr d);
  virtual int put_data(int x, int y, KRGB d);
  virtual int put_data(int x, int y, int col);
	
  unsigned char *get_pBMP(void) { return data; }
	
protected:
  int getPos(int x, int y) {
    return ((sizeV - y - 1) * byte_sizeH + (int)(x / 2));
  }
  int checkColorNum(int n);
};

#endif	//	end of kim_data.h
