#include <basic.h>
#include <new>
#include "kpalet.h"
#include "ketc.h"

//	KPallet class functions

KPallet::KPallet(int n)
{
  palletRGB = new(std::nothrow) KRGB[n];
  palletYPbPr = new(std::nothrow) KYPbPr[n];
  pal_num = n;
}

KPallet::~KPallet()
{
  delete [] palletRGB;
  delete [] palletYPbPr;
}

int KPallet::chkNum(int n)
{
  if ((n < 0) || (n >= pal_num)){
    return 1;
  }
  return 0;
}

KRGB KPallet::get_color(int n)
{
  if (chkNum(n) != 0){
    return KRGB(-1, -1, -1);
  }
  return palletRGB[n];
}

int KPallet::put_color(int n, KRGB d)
{
  if (chkNum(n) != 0){
    return 1;
  }
  palletRGB[n] = d;
  palletYPbPr[n] = d.get_YPbPr();
  return 0;
}

int KPallet::put_color(int n, KYPbPr d)
{
  if (chkNum(n) != 0){
    return 1;
  }
  palletRGB[n] = d.get_RGB();
  palletYPbPr[n] = d;
  return 0;
}

int KPallet::srch_color(KRGB d)
{
  for (int i = 0; i < pal_num; i++){
    if ((palletRGB[i].RR == d.RR)
        && (palletRGB[i].GG == d.GG)
        && (palletRGB[i].BB == d.BB)){
      return i;
    }
  }
  return -1;
}

int KPallet::srch_nearRGBColor(KRGB d)
{
  unsigned int min_ln = 0xffffffff;
  int min_num = 0;
  unsigned int ln;
  int rl, gl, bl;
	
  clipping(d.RR, 0, 255);
  clipping(d.GG, 0, 255);
  clipping(d.BB, 0, 255);
  for (int i = 0; i < pal_num; i++){
    rl = palletRGB[i].RR - d.RR;
    gl = palletRGB[i].GG - d.GG;
    bl = palletRGB[i].BB - d.BB;
    // B-right/VǤ32ӥåȤʤΤǡ֤
    // rl /= 10;	gl /= 10;	bl /= 10;
    ln = rl * rl + gl * gl + bl * bl;
    if (ln == 0){
      return i;
    }
    if (ln < min_ln){
      min_ln = ln;
      min_num = i;
    }
  }
  
  return min_num;
}

int KPallet::srch_nearYPbPrColor(KYPbPr d)
{
  unsigned int min_ln = 0xffffffff;
  int min_num = 0;
  unsigned int ln;
  int yl, pbl, prl;
	
  clipping(d.YY, 0, 255);
  clipping(d.Pb, -128, 127);
  clipping(d.Pr, -128, 127);
  for (int i = 0; i < pal_num; i++){
    yl  = (palletYPbPr[i].YY - d.YY);
    pbl = (palletYPbPr[i].Pb - d.Pb);
    prl = (palletYPbPr[i].Pr - d.Pr);
//		yl  = (palletYPbPr[i].YY - d.YY) / 2;
//		pbl = (palletYPbPr[i].Pb - d.Pb) * 2;
//		prl = (palletYPbPr[i].Pr - d.Pr) * 2;
    // B-right/VǤ32ӥåȤʤΤǡ֤
    // yl /= 10;	pbl /= 10;	prl /= 10;
    ln = yl * yl + pbl * pbl + prl * prl;
    if (ln == 0){
      return i;
    }
    if (ln < min_ln){
      min_ln = ln;
      min_num = i;
    }
  }
  return min_num;
}

/*
void main(void)
{
	KPallet test(16);
	for(int i = 0; i < 16; i++){
		test.put_color(i, KRGB(i, i, i));
	}
	for(i = 0; i < 17; i++){
		KRGB buff = test.get_color(i);
		buff.prt_data();
	}
	cout << "pallet no = " << test.srch_color(KRGB(5, 5, 5)) << endl;
	cout << "pallet no = " << test.srch_nearYPbPrColor(KYPbPr(5.1, 5, 5.3)) << endl;
}
*/


