#include <basic.h>
#include <bstdio.h>
#include <btron/dp.h>
#include <btron/file.h>
#include <tcode.h>
#include <tad.h>
#include <new>
#include "record.h"
#include "r_obj.h"

//#define READ_FLAGS (F_READ|F_EXCL)
#define READ_FLAGS F_READ // -- ǥХåϤäˤʤ

#define SEGMENT_BUFF_SIZE		100
#define SEGMENT_UNBUFF_SIZE		100

Segment::Segment(TC *str)
{
  segData = new(std::nothrow) KText;
  segData->setLine(str);
  segID = str[0] & 0xff;
  length = 0;
  vObj = NULL;
  Buff = new(std::nothrow) TC[SEGMENT_BUFF_SIZE];
  unBuff = new(std::nothrow) TC[SEGMENT_UNBUFF_SIZE];
  pGet = 0;
  punGet = 0;
}

Segment::Segment()
{
  segData = new(std::nothrow) KText;
  segID = 0;
  vObj = NULL;
  Buff = new(std::nothrow) TC[SEGMENT_BUFF_SIZE];
  unBuff = new(std::nothrow) TC[SEGMENT_UNBUFF_SIZE];
  pGet = 0;
  punGet = 0;
}

Segment::~Segment()
{
// b_printf("Segment destractor \n");
  del_data();
  delete[] Buff;
  delete[] unBuff;
}

LINK* Segment::get_vobj()
{
  if (segID == TS_VOBJ){
    if (vObj == NULL){
      return NULL;
    }
    return vObj->getVirtualObject();
  }
  return NULL;
}

B* Segment::get_seg(B* str)
{
  return str;
}

TC Segment::getw()
{
  if (segData == NULL){
    return EOF;
  }
	
  TC c;

  if (punGet < 0){
    c = unBuff[-punGet];
    punGet++;
  }
  else {
    if (Buff[pGet] == TNULL){
      if (getBuff() == EOF){
        return EOF;
      }
    }
    c = Buff[pGet++];
  }

  return c;
}

TC Segment::ungetw(TC c)
{
  if (punGet < -(SEGMENT_UNBUFF_SIZE - 2)){
    return EOF;
  }
  punGet--;
  unBuff[-punGet] = c;
	
  return c;
}

int Segment::getBuff()
{
  pGet = 0;
  return segData->getTextLine(Buff);
}



Fusen::Fusen(TC *str, int n)
  : Segment()
{
  length = n;
  if (Buff != NULL){
    delete [] Buff;
  }
  Buff = new(std::nothrow) TC[n+1];
  for (int i = 0; i < n; i++){
    Buff[i] = str[i];
  }
  Buff[n] = TNULL;
  segID = str[0] & 0xff;
}

Fusen::~Fusen()
{
// b_printf("Fusen destructor \n");
}

B* Fusen::get_seg(B* str)
{
  if (str != NULL){
    delete [] str;
  }
  str = new(std::nothrow) B[length*2];
  memcpy(str, (B*)Buff, length*2);
	
  return str;
}

TC Fusen::getw()
{
  if (segData == NULL){
    return EOF;
  }
	
  TC c;
	
  if (punGet < 0){
    c = unBuff[-punGet];
    punGet++;
  }
  else {
    if (pGet < length){
      c = Buff[pGet++];
    }
    else {
      c = EOF;
    }
  }
  return c;
}



TADMainRecord::TADMainRecord(LINK* lnk, RealObject *rp)
{
  parent = rp;
  plink = lnk;
  seg = NULL;
  recBuff = NULL;
  fd = -1;
  read_rec();
}

TADMainRecord::~TADMainRecord()
{
  close_file();
  del_seg();
}

void TADMainRecord::close_file()
{
  if (fd >= 0){
    cls_fil(fd);
    fd = -1;
  }
  if (recBuff != NULL){
    delete recBuff;
    recBuff = NULL;
  }
}

Segment* TADMainRecord::get_segment()
{
  read_seg();
  seg->getBuff();
  return seg;
}

int TADMainRecord::read_seg()
{
  del_seg();
  TC buf[BUFFER_SIZE];
  TC *p = &buf[0];
  TC data = recBuff->getw();
  int len = 0;
	
  if (data >= 0xff00){
// b_printf("FUSEN SEGMENT \n");
    *p++ = data;
    len = get_seglen();
    for (int i = 0; i < len+1; i++){
      data = recBuff->getw();
      *p++ = data;
    }
    *p = TNULL;
    seg = new(std::nothrow) Fusen(buf, len+2);
    if (seg->getKind() == TS_VOBJ){
      VirtualObject *lp = parent->getVirtualObject();
      if (lp != NULL){
        seg->set_vobj(lp);
      }
    }
    return (len+2);
  }
  else {
//b_printf("DATA SEGMENT \n");
    seg = new(std::nothrow) Segment();
    while (data < 0xff00){
      *p++ = data;
      len++;
      if (data == TK_NL){
        *p = TNULL;
        seg->set_data(buf);
        p = &buf[0];
      }
      data = recBuff->getw();
    }
  }
  recBuff->ungetw(data);
  *p = TNULL;
  if (tc_strlen(buf) != 0){
    seg->set_data(buf);
  }
  seg->set_len(len);
  return len;
}

int TADMainRecord::get_seglen()
{
  TC len = recBuff->getw();
  recBuff->ungetw(len);
  return (len / 2);
}

int TADMainRecord::read_rec()
{
  close_file();
  fd = opn_fil(plink, READ_FLAGS, NULL);
  if (fd < 0){
//		b_printf("TADMainRecord::read_rec can't open file \n");
    return -1;
  }
  recBuff = new(std::nothrow) RecordBuffer(fd);
	
  return 0;
}




RealObject::RealObject(LINK* lnk)
{
//b_printf("RealObject : constructor end\n");
  plink = lnk;
  vobjList = new(std::nothrow) VirtualObjectList;
  read_vobj();
  mainRec = new(std::nothrow) TADMainRecord(lnk, this);
  fil_sts(plink, name, NULL, NULL);
}

RealObject::~RealObject()
{
  delete vobjList;
  if (mainRec != NULL){
    delete mainRec;
  }
}

int RealObject::read_vobj()
{
  vobjList->deleteAll();
  W fd = opn_fil(plink, READ_FLAGS, NULL);
  if (fd < 0){
//b_printf("RealObject::read_vobj Can't open file\n");
    return -1;
  }
	
  if (fnd_rec(fd, F_TOPEND, 0x00000001, 0, NULL) == 0){
    do {
      LINK dummy;
      W recSize;
      UH subType;
      rea_rec(fd, 0, (B*)&dummy, sizeof(LINK), &recSize, &subType);
      vobjList->setObject(&dummy);
    } while (fnd_rec(fd, F_NFWD, 0x00000001, 0, NULL) == 0);
  }
  cls_fil(fd);
  vobjList->seekTop();
  return 0;
}
